import logging

from typing import Dict


log = logging.getLogger(__name__)


threading_local = None
try:
    from log_request_id import local

    threading_local = local
except ImportError:
    log.warning("Can't check request_id", exc_info=True)


def get_request_id_header() -> Dict[str, str]:
    if not threading_local:
        return {}
    request_id = getattr(threading_local, "request_id", None)
    if not request_id or request_id == "none":
        return {}
    return {"X-Request-Id": request_id}



import enum

from typing import List, Optional, Union
from urllib.parse import urljoin

from pydantic import BaseModel, validator

from sdk import SDKClient
from sdk.client import SDKResponse


class BalancePenalty(BaseModel):
    hour: int
    stop: int


class BalanceGroup(BaseModel):
    id: str
    penalty: BalancePenalty


class DeliveryOptions(BaseModel):
    # https://yandex.ru/routing/doc/vrp/concepts/properties-of-vehicles.html#routing-mode
    routing_mode: str = "transit"
    time_zone: str = 3
    date: str
    # https://yandex.ru/routing/d/search/?query=global_proximity_factor
    global_proximity_factor: float = 2.0
    # https://yandex.ru/routing/doc/vrp/concepts/balanced-groups.html?lang=ru
    balanced_groups: Optional[List[BalanceGroup]]


class Point(BaseModel):
    lat: float = 55.781053
    lon: float = 37.599911


class Depot(BaseModel):
    id: int = 0
    ref: str = "Новослободская 14/19с1"
    point: Point = Point()
    time_window: str = "08:00:00-20:00:00"


class StopExcessPenalty(BaseModel):
    per_stop: int = 100


class ShiftPenalty(BaseModel):
    stop_excess: StopExcessPenalty = StopExcessPenalty()


class Shift(BaseModel):
    hard_window: bool = False
    time_window: str = "08:00:00-20:00:00"
    id: str = "0"
    balanced_group_id: Optional[str]
    maximal_stops: Optional[int]
    penalty: Optional[ShiftPenalty]


class VehicleCapacity(BaseModel):
    units: int = 85


class Vehicle(BaseModel):
    id: str
    shifts: List[Shift]
    return_to_depot: bool = True
    finish_at: Optional[int]
    # https://yandex.ru/routing/doc/vrp/concepts/properties-of-vehicles.html#shift-stops
    capacity: VehicleCapacity = VehicleCapacity()
    # https://yandex.ru/routing/doc/vrp/concepts/properties-of-vehicles.html#tags
    tags: Optional[List[str]]


class ShipmentSize(BaseModel):
    units: int


class LocationType(enum.Enum):
    delivery = "delivery"
    garage = "garage"
    pickup = "pickup"


class Location(BaseModel):
    id: int
    title: str
    type: LocationType = LocationType.delivery
    time_window: str = "08:00:00-20:00:00"
    point: Point
    description: str = ""
    shipment_size: Optional[ShipmentSize]
    required_tags: Optional[List[str]]


class DeliveryRequest(BaseModel):
    depot: Depot
    vehicles: List[Vehicle]
    locations: List[Location]
    options: DeliveryOptions


class DeliveryResponse(BaseModel):
    id: str
    message: str


class Courier(BaseModel):
    company_id: int
    id: int
    name: str
    number: str
    phone: str
    sms_enabled: bool


class RouteStatus(BaseModel):
    completed: Optional[float]


class RouteDepotValue(BaseModel):
    id: int
    point: Point
    routing_mode: str
    time_window: str
    ref: str


class RouteLocationValue(BaseModel):
    id: int
    description: str
    title: str
    point: Point
    routing_mode: str
    time_window: str


class RouteNodeType(enum.Enum):
    depot = "depot"
    location = "location"


class RouteNode(BaseModel):
    type: RouteNodeType
    used_time_window: str
    value: Union[RouteLocationValue, RouteDepotValue]


class RouteLocation(BaseModel):
    node: RouteNode


class Route(BaseModel):
    run_number: int
    vehicle_id: str
    route: List[RouteLocation]


OVERLOAD_DROP_REASON = "Перегрузка"


class DroppedRoute(BaseModel):
    id: int
    drop_reason: str

    @validator("drop_reason")
    def prepare_drop_reason(cls, v):
        if "Vehicle overload" in v:
            return OVERLOAD_DROP_REASON
        return v


class RouteResult(BaseModel):
    routes: List[Route]
    vehicles: List[Vehicle]
    dropped_locations: List[DroppedRoute]


class RouteInfo(BaseModel):
    id: str
    status: RouteStatus
    message: str
    result: RouteResult


class GetRoute(BaseModel):
    id: str


class YandexCourierService:
    def __init__(
        self,
        client: SDKClient,
        delivery_token: str,
        dictionaries_token: str,
        company_id: int,
    ):
        self._client = client
        self._delivery_url = "https://courier.yandex.ru/vrs/api/v1/"
        self._dict_url = f"https://courier.yandex.ru/api/v1/companies/{company_id}/"
        self._delivery_token = delivery_token
        self._dictionaries_token = dictionaries_token

    def create_delivery(
        self, query: DeliveryRequest, timeout: int = 3
    ) -> SDKResponse[DeliveryResponse]:
        return self._client.post(
            urljoin(str(self._delivery_url), "add/mvrp"),
            response_class=DeliveryResponse,
            params={"apikey": self._delivery_token},
            data=query.json(exclude_none=True),
            timeout=timeout,
        )

    def get_delivery(self, query: GetRoute, timeout: int = 3) -> SDKResponse[RouteInfo]:
        return self._client.get(
            urljoin(str(self._delivery_url), f"result/mvrp/{query.id}"),
            response_class=RouteInfo,
            params={"apikey": self._delivery_token},
            data=query.json(),
            timeout=timeout,
        )

    def get_couriers(self, timeout=3) -> SDKResponse[List[Courier]]:
        return self._client.get(
            urljoin(str(self._dict_url), "couriers"),
            response_class=Courier,
            headers={"Authorization": f"Auth {self._dictionaries_token}"},
            timeout=timeout,
        )



import datetime

from enum import IntEnum
from typing import List, Literal, Optional, Union

from pydantic import BaseModel, Field

from .base import PdfResponse
from .client import Empty, HttpUrl, SDKClient, SDKResponse


class OrderType(IntEnum):
    lmk = 0
    prof = 1
    lab = 2
    cert = 3


class ExamType(IntEnum):
    preliminary = 1
    periodical = 2


class Payment(IntEnum):
    self_pay = 1  # платит клиент
    org_pay = 2  # платит организация


class HealthGroup(IntEnum):
    first = 1
    second = 2
    third = 3


class NsiMap(BaseModel):
    code: str
    name: str


class Factor(BaseModel):
    code_29n: str  # nsi_id now


class FiasAddress(BaseModel):
    ao_guid: str
    house_guid: Optional[str] = None

    class Config:
        use_enum_values = True


class Address(BaseModel):
    full: str
    state_code: str
    postal_code: Optional[str] = None
    fias: Optional[FiasAddress] = None

    class Config:
        use_enum_values = True


class AddPatientRequest(BaseModel):
    uuid: str
    fname: str
    mname: Optional[str]
    lname: str
    birth: datetime.date
    sex: str

    passport_type: Optional[str]
    passport_number: Optional[str]
    passport_series: Optional[str]
    passport_date: Optional[datetime.date]
    passport_department_code: Optional[str]

    snils: str

    phone: Optional[str]
    email: Optional[str]

    quarter: Optional[str]
    building: Optional[str]
    city: Optional[str]
    street: Optional[str]

    registration_quarter: Optional[str]
    registration_building: Optional[str]
    registration_city: Optional[str]
    registration_street: Optional[str]
    mc_id: int

    @property
    def full_address(self):
        return " ".join(
            a
            for a in [
                self.city,
                self.street,
                self.building,
                self.quarter,
            ]
            if a
        )

    @property
    def registration_address(self):
        return " ".join(
            a
            for a in [
                self.registration_city,
                self.registration_street,
                self.registration_building,
                self.registration_quarter,
            ]
            if a
        )


class MedCenter(BaseModel):
    id: str
    license: str
    license_registration: Optional[str]
    ogrn: str
    okpo: str
    legalname: str
    phone: str
    legaladdress: str
    zip_code: str
    email: str


class Lmk(BaseModel):
    number: Optional[str] = None
    is_preliminary: bool


class Order(BaseModel):
    number: str
    exams_start: datetime.datetime
    exams_done: datetime.datetime
    next_date: datetime.date
    org_name: Optional[str]
    org_inn: Optional[str]
    post: Optional[str] = None
    factors: List[Factor]
    payment: Optional[Payment]
    code_unit: str = ""
    conclusion: Optional[str] = None
    exam_type: Optional[ExamType]
    health_group: Optional[HealthGroup]
    lmk: Optional[Lmk]

    class Config:
        use_enum_values = True


class Doctor(BaseModel):
    mis_id: str
    lname: str
    fname: str
    mname: Optional[str]
    snils: Optional[str]
    address: Optional[Address]
    speciality: Optional[str]
    post: Optional[str]
    position: Optional[NsiMap]

    class Config:
        use_enum_values = True


class Exam(BaseModel):
    conclusion: str
    date: datetime.date
    doctor: Doctor
    id: str
    nsi_id: Optional[str] = None


class LabService(BaseModel):
    date: datetime.date
    result: Optional[str]
    name: str
    id: str
    nsi_id: Optional[str] = None


class FuncDiag(BaseModel):
    date: datetime.date
    conclusion: str
    name: str
    id: str
    nsi_id: Optional[str] = None


class Vaccine(BaseModel):
    name: str
    step: Optional[str] = None
    date: datetime.date
    is_revac: bool = False
    nsi_id: Optional[str] = None


class Infection(BaseModel):
    year: int
    mkb: str


class AddMedRecordRequest(BaseModel):
    mc_id: int
    order_type: OrderType
    order: Order
    creation_date: datetime.datetime  # lmk - order_time / prof - exams_start
    patient: AddPatientRequest
    author: Doctor
    exams: List[Exam] = []
    lab_services: List[LabService] = []
    functional_diagnostics: List[FuncDiag] = []
    vaccinations: List[Vaccine] = []
    infections: List[Infection] = []
    samd_type: Literal["103", "194", "230"] = Field(
        description="Доступные типы документов samd"
    )

    class Config:
        use_enum_values = True


class SamdResponse(BaseModel):
    task_id: str


class SamdPDFRequest(BaseModel):
    task_id: Optional[int] = None
    order_number: Optional[str] = ""


class SamdService:
    def __init__(self, client: SDKClient, url: HttpUrl):
        self._client = client
        self._url = url
        self.add_patient_url = self._url + "/api/patient/"
        self.add_med_record_url = self._url + "/api/med_record/"

    def add_patient(self, query: AddPatientRequest, timeout=3) -> SDKResponse[Empty]:
        return self._client.post(
            self.add_patient_url,
            SamdResponse,
            data=query.json(),
            headers={"Content-Type": "application/json"},
            timeout=timeout,
        )

    def add_med_record(
        self, query: AddMedRecordRequest, timeout=3
    ) -> SDKResponse[Empty]:
        return self._client.post(
            self.add_med_record_url,
            SamdResponse,
            data=query.json(),
            headers={"Content-Type": "application/json"},
            timeout=timeout,
        )

    def med_record_pdf(
        self, query: SamdPDFRequest, timeout=3
    ) -> SDKResponse[Union[PdfResponse, Empty]]:
        return self._client.get(
            self._url + "api/med_record/pdf/",
            PdfResponse,
            params=query.dict(),
            headers={"Content-Type": "application/pdf"},
            timeout=timeout,
        )



from enum import Enum

from pydantic import BaseModel


class Gender(str, Enum):
    MALE = "M"
    FEMALE = "F"


class BytesResponse(BaseModel):
    bytes: bytes


class PdfResponse(BytesResponse):
    pass


class SdkException(Exception):
    pass



import datetime
import math

from enum import Enum
from typing import List, Optional, Union

from pydantic import BaseModel, Field, HttpUrl

from .client import SDKClient, SDKResponse, Empty


class ServiceCode(Enum):
    lmk = "lmk"
    lab = "lab"


class AmountType(Enum):
    fix = "fix"
    percent = "percent"


class GiftCodesRequest(BaseModel):
    code: str = None
    not_expired: bool = True
    is_promo: bool = True


class GiftCodeRequest(BaseModel):
    code: str


class GenerateCodeRequest(BaseModel):
    amount: Optional[int]
    amount_type: Optional[AmountType]
    cost: Optional[int]

    class Config:
        use_enum_values = True


class UseCode(BaseModel):
    code: str
    service: ServiceCode
    phone: Optional[str] = Field(description="Телефон клиента, применяющего промокод")
    object_id: Optional[str] = Field(
        description="Идентификатор заказа, к которому применяется промокод в системе источника"
    )
    order_id: Optional[str] = Field(
        description="Идентификатор заявки/предварительной записи связанной с заявкой. Без него промокод арендуется на краткое время"
    )
    source: str = Field(description="Источник, может быть любым текстом")
    cost: Optional[int] = Field(description="Цена без применения промокодов")
    visit_at: Optional[datetime.date] = Field(
        description="Нужна для проверки действителен, ли промокод на дату визита"
    )

    class Config:
        use_enum_values = True


class UnUseCode(BaseModel):
    code: str
    object_id: str

    class Config:
        use_enum_values = True


class GiftCodeResponse(BaseModel):
    code: str
    amount: int
    amount_type: AmountType
    dc: datetime.datetime
    expired_date: datetime.datetime
    used_date: Optional[datetime.datetime]
    is_reusable: bool
    is_promo: bool
    is_first_time: bool
    min_cost: Optional[int]

    class Config:
        use_enum_values = True

    def get_discount(self, cost: int) -> int:
        """Округляем до целого числа делящегося на 5 в сторону клиента,
        по указанию от Леши Пермякова"""
        if self.amount_type == AmountType.percent.value:
            return math.ceil(cost * self.amount / 100 / 5) * 5
        return self.amount


class GiftError(BaseModel):
    code: str
    message: str


class GiftCodeCheckResponse(GiftCodeResponse):
    errors: List[GiftError]


class GiftService:
    def __init__(self, client: SDKClient, url: HttpUrl):
        self._client = client
        if not url.endswith("/"):
            url = f"{url}/"
        self._url = url

    def get_codes(
        self, query: GiftCodesRequest = GiftCodesRequest(), timeout=3
    ) -> SDKResponse[List[GiftCodeResponse]]:
        return self._client.get(
            self._url + "gifts/rest/",
            GiftCodeResponse,
            params=query.dict(exclude_none=True),
            timeout=timeout,
        )

    def get_code(
        self, query: GiftCodeRequest, timeout=3
    ) -> SDKResponse[GiftCodeResponse]:
        params = query.dict(exclude_none=True)
        code = params.pop("code")
        return self._client.get(
            self._url + f"gifts/rest/{code}/",
            GiftCodeResponse,
            params=params,
            timeout=timeout,
        )

    def use(self, query: UseCode, timeout: int = 3) -> SDKResponse[GiftCodeResponse]:
        params = query.dict(exclude_none=True)
        code = params.pop("code")
        return self._client.patch(
            self._url + f"gifts/rest/{code}/use/",
            GiftCodeResponse,
            data=params,
            timeout=timeout,
        )

    def un_use(
        self, query: UnUseCode, timeout: int = 3
    ) -> SDKResponse[GiftCodeResponse]:
        params = query.dict(exclude_none=True)
        code = params.pop("code")
        return self._client.patch(
            self._url + f"gifts/rest/{code}/un_use/",
            GiftCodeResponse,
            data=params,
            timeout=timeout,
        )

    def check(
        self, query: UseCode, timeout: int = 3
    ) -> SDKResponse[GiftCodeCheckResponse]:
        params = query.dict(exclude_none=True)
        code = params.pop("code")
        return self._client.get(
            self._url + f"gifts/rest/{code}/check/",
            GiftCodeCheckResponse,
            params=params,
            timeout=timeout,
        )

    def generate(
        self, query: GenerateCodeRequest, timeout: int = 3
    ) -> SDKResponse[Union[GiftCodeResponse, Empty]]:
        return self._client.post(
            self._url + "gifts/rest/generate",
            GiftCodeResponse,
            data=query.dict(exclude_none=True),
            timeout=timeout,
        )



from urllib.parse import urljoin

from pydantic import BaseModel

from .client import Empty, SDKClient, SDKResponse


class NoteStatusRequest(BaseModel):
    ip: str
    have_local_changes: bool


class NoteLogRequest(BaseModel):
    task: int
    status: str
    description: str


class NoteMasterService:
    def __init__(self, client: SDKClient):
        self._client = client

    def status(
        self, base_url: str, query: NoteStatusRequest, timeout=3
    ) -> SDKResponse[Empty]:
        return self._client.post(
            urljoin(base_url, "status/"), Empty, data=query.dict(), timeout=timeout,
        )

    def create_log(
        self, base_url: str, query: NoteLogRequest, timeout=3
    ) -> SDKResponse[Empty]:
        return self._client.post(
            urljoin(base_url, "log/"), Empty, data=query.dict(), timeout=timeout
        )



import datetime

from enum import Enum
from typing import List, Optional
from urllib.parse import urljoin

from pydantic import BaseModel, Field, HttpUrl, validator

from ..base import PdfResponse
from ..client import Empty, SDKClient, SDKResponse


class Client(BaseModel):
    phone_number: Optional[str]
    email: Optional[str]
    last_name: str
    first_name: str
    middle_name: Optional[str]
    birth: str
    post: Optional[str] = Field(alias="job")


class Trace(BaseModel):
    uuid: str
    label: str
    dt: Optional[datetime.date]
    description: str

    @validator("dt", pre=True)
    def ignore_time(cls, value):
        if value:
            return datetime.datetime.strptime(value[:10], "%Y-%m-%d").date()
        return


class LmkTraceResponse(BaseModel):
    client: Client
    trace: List[Trace]


class Medicine(BaseModel):
    latest_date: datetime.date
    next_date: Optional[datetime.date]
    nearest_expired_date: Optional[datetime.date]
    has_admission: Optional[bool]


class Attestation(BaseModel):
    latest_date: Optional[datetime.date]
    next_date: Optional[datetime.date]
    reg_date: Optional[datetime.date]


class Lmk(BaseModel):
    reg_number: Optional[str]
    blank_number: str
    med_dat: Optional[datetime.date]
    job: str
    category: str
    type: str
    need_att: bool


class LmkError(str, Enum):
    not_all_medicine = "not_all_medicine"
    attestation_not_found = "attestation_not_found"


class LmkServiceResult(BaseModel):
    name: str
    priority: float
    result_date: Optional[datetime.date]
    expired_date: Optional[datetime.date]
    done: bool
    was: bool
    is_external: bool


class LmkDelivery(BaseModel):
    ready_date: Optional[datetime.date]
    hand_date: Optional[datetime.date]


class Order(BaseModel):
    date: datetime.date
    number: str
    mc: str


class LmkProblem(BaseModel):
    name: str
    code: str
    is_notified: bool


class CheckLmkResponse(BaseModel):
    lmk: Lmk
    medicine: Medicine
    attestation: Optional[Attestation]
    client: Client
    warnings: List[LmkError]
    problems: List[LmkProblem]
    medicine_results: Optional[List[LmkServiceResult]]
    delivery: Optional[LmkDelivery]
    last_order: Optional[Order]

    class Config:
        use_enum_values = True


class OrderTraceRequest(BaseModel):
    phone: str
    lab_number: str


class CheckLmkRequest(BaseModel):
    blank_number: str
    last_name: str
    first_name: str
    take_medicine: bool = False


class SetHandDateRequest(BaseModel):
    uuids: List[str]
    hand_date: datetime.date


class ConclusionRequest(BaseModel):
    lab_number: int


class LmkService:
    def __init__(self, client: SDKClient, url: HttpUrl = None):
        self._client = client
        self._url = url

    def order_trace(
        self, query: OrderTraceRequest, timeout=3
    ) -> SDKResponse[LmkTraceResponse]:
        return self._client.get(
            urljoin(str(self._url), "lmk/rest/order_detailed_trace_step/"),
            LmkTraceResponse,
            params=query.dict(),
            timeout=timeout,
        )

    def check_lmk(
        self, query: CheckLmkRequest, timeout=3
    ) -> SDKResponse[CheckLmkResponse]:
        return self._client.get(
            urljoin(str(self._url), "lmk/rest/check_lmk_v2/"),
            CheckLmkResponse,
            params=query.dict(),
            timeout=timeout,
        )

    def set_orders_hand_date(
        self, query: SetHandDateRequest, mis_url, timeout=3
    ) -> SDKResponse[Empty]:
        return self._client.post(
            urljoin(mis_url, "lmk/rest/orders/set_hand_date/"),
            Empty,
            data=query.dict(),
            timeout=timeout,
        )

    def conclusion_pdf(
        self, query: ConclusionRequest, timeout=30
    ) -> SDKResponse[PdfResponse]:
        return self._client.get(
            urljoin(str(self._url), f"lmk/api/conclusion/{query.lab_number}/"),
            PdfResponse,
            headers={"Content-Type": "application/pdf"},
            timeout=timeout,
        )



from typing import List, Optional

from pydantic import BaseModel

from .client import HttpUrl, SDKClient, SDKResponse


class ResponseResult(BaseModel):
    code: str
    messageId: int


class DevinoResponse(BaseModel):
    code: str
    description: Optional[str]
    result: ResponseResult


class VkData(BaseModel):
    subject: str
    priority: str
    routes: List[str]
    validityPeriod: int
    phone: str
    templateId: str
    templateData: dict


class SendVkRequest(BaseModel):
    vk: VkData


class ViberData(BaseModel):
    subject: str
    priority: str
    validityPeriod: int
    type: str = "viber"
    contentType: str
    text: str
    dstAddress: str


class SmsData(BaseModel):
    srcAddress: str
    text: str
    validityPeriod: int
    dstAddress: str


class SendAllRequest(BaseModel):
    vk: VkData
    viber: ViberData
    sms: SmsData


class DevinoService:
    def __init__(self, client: SDKClient, url: HttpUrl, token: str):
        self.client = client
        self._url = url
        self.token = token
        self.send_vk_url = self._url + "send/vk"

    def send_vk(self, query: SendVkRequest, timeout=3) -> SDKResponse[DevinoResponse]:
        return self.client.post(
            self.send_vk_url,
            DevinoResponse,
            data=query.json(),
            headers={
                "Authorization": f"Basic {self.token}",
                "Content-Type": "application/json",
            },
            timeout=timeout,
        )

    def send_all(self, query: SendAllRequest, timeout=3) -> SDKResponse[DevinoResponse]:
        return self.client.post(
            self.send_vk_url,
            DevinoResponse,
            data=query.json(),
            headers={
                "Authorization": f"Basic {self.token}",
                "Content-Type": "application/json",
            },
            timeout=timeout,
        )



from typing import List, Optional

from pydantic import Field, BaseModel, validator

from .client import SDKClient, SDKResponse


class NsiIdentifier(BaseModel):
    identifier: str


class NsiQuery(NsiIdentifier):
    version: str
    size: int = 200


class NsiResponse(BaseModel):
    result: Optional[str]
    resultText: Optional[str] = None
    resultCode: Optional[str] = None
    total: Optional[int] = None
    data: Optional[list] = Field(alias="list")
    exception: Optional[str] = None
    message: Optional[str] = None

    @validator("data", pre=True)
    def transform_data(cls, items):
        """Преобразуем eav формат в набор полей необходимых для справочника."""
        if not isinstance(items, list):
            return items
        transformed = []
        for row in items:
            transformed_item = {}
            for column in row:
                name = column["column"].lower()
                if name == "id":
                    transformed_item["nsi_id"] = column["value"]
                else:
                    transformed_item[name] = column["value"]
            transformed.append(transformed_item)
        return transformed


class NsiVersionResponse(BaseModel):
    result: Optional[str]
    resultText: Optional[str] = None
    resultCode: Optional[str] = None
    identifier: Optional[str]
    oid: Optional[str]
    version: Optional[str]
    rowsCount: Optional[int]
    lastUpdate: Optional[str]
    approveDate: Optional[str]
    exception: Optional[str] = None
    message: Optional[str] = None


class NsiService:
    def __init__(self, client: SDKClient, token: str) -> None:
        self._client = client
        self._url_data = "https://nsi.rosminzdrav.ru/port/rest/data/"
        self._url_passport = "https://nsi.rosminzdrav.ru/port/rest/passport"
        self._token = token

    def get_data(
        self, query: NsiQuery, timeout=3
    ) -> SDKResponse[NsiResponse]:
        return self._client.get(
            self._url_data,
            NsiResponse,
            params={
                "identifier": query.identifier,
                "size": query.size,
                "userKey": self._token,
                "version": query.version,
            },
            timeout=timeout,
        )

    def check_version(
        self, query: NsiIdentifier, timeout=3
    ) -> SDKResponse[NsiVersionResponse]:
        return self._client.get(
            self._url_passport,
            NsiVersionResponse,
            params={
                "identifier": query.identifier,
                "userKey": self._token,
            },
            timeout=timeout,
        )



import io

from typing import List, Optional

from pydantic import BaseModel, HttpUrl

from .client import Empty, SDKClient, SDKResponse


class FaceDetectorDetectRequest(BaseModel):
    external_id: str
    file: bytes
    file_name: str


class FindSimilarFaceRequest(BaseModel):
    file_name: str
    file: bytes


class FindSimilarFaceResponse(BaseModel):
    external_id: str
    distance: float
    comment: Optional[str]
    image: Optional[bytes]


class FaceDetectorService:
    def __init__(self, client: SDKClient, url: HttpUrl, token: str):
        self._client = client
        self._token = token
        self._url = url
        self._url_detect = self._url + "api/get_face_descriptor/"
        self._url_similar = self._url + "api/find_similar/"

    def detect(self, query: FaceDetectorDetectRequest, timeout=3) -> SDKResponse[Empty]:
        file = {"image": (query.file_name, io.BytesIO(query.file))}
        headers = {"Authorization": f"Token {self._token}"}
        return self._client.post(
            self._url_detect,
            Empty,
            data={"external_id": query.external_id},
            files=file,
            timeout=timeout,
            headers=headers,
        )

    def find_similar(
        self, query: FindSimilarFaceRequest, timeout=3
    ) -> SDKResponse[List[FindSimilarFaceResponse]]:
        file = {"image": (query.file_name, io.BytesIO(query.file))}
        headers = {"Authorization": f"Token {self._token}"}
        return self._client.post(
            self._url_similar,
            FindSimilarFaceResponse,
            files=file,
            timeout=timeout,
            headers=headers,
        )
